@@ -38,9 +38,11 @@ THE SOFTWARE.
 #include <string.h>
 #include <math.h>
 #include <stdint.h>
+#include <fcntl.h>
 #include <limits.h>
 #include <assert.h>
 #include <arpa/inet.h>
+#include <sys/stat.h>
 
 #include <libavcodec/avcodec.h>
 #include <libavutil/avutil.h>
@@ -254,6 +256,15 @@ static bool openStream (player_t * const player) {
 	return true;
 }
 
+/*	Get output sample rate. Default to stream sample rate
+ */
+static int getSampleRate (const player_t * const player) {
+	AVCodecParameters const * const cp = player->st->codecpar;
+	return player->settings->sampleRate == 0 ?
+			cp->sample_rate :
+			player->settings->sampleRate;
+}
+
 /*	setup filter chain
  */
 static bool openFilter (player_t * const player) {
@@ -289,8 +300,8 @@ static bool openFilter (player_t * const player) {
 
 	/* aformat: convert float samples into something more usable */
 	AVFilterContext *fafmt = NULL;
-	snprintf (strbuf, sizeof (strbuf), "sample_fmts=%s",
-			av_get_sample_fmt_name (avformat));
+	snprintf (strbuf, sizeof (strbuf), "sample_fmts=%s:sample_rates=%d",
+			av_get_sample_fmt_name (avformat), getSampleRate (player));
 	if ((ret = avfilter_graph_create_filter (&fafmt,
 					avfilter_get_by_name ("aformat"), "format", strbuf, NULL,
 					player->fgraph)) < 0) {
@@ -328,13 +339,33 @@ static bool openDevice (player_t * const player) {
 	aoFmt.bits = av_get_bytes_per_sample (avformat) * 8;
 	assert (aoFmt.bits > 0);
 	aoFmt.channels = cp->channels;
-	aoFmt.rate = cp->sample_rate;
+	aoFmt.rate = getSampleRate (player);
 	aoFmt.byte_format = AO_FMT_NATIVE;
 
-	int driver = ao_default_driver_id ();
-	if ((player->aoDev = ao_open_live (driver, &aoFmt, NULL)) == NULL) {
-		BarUiMsg (player->settings, MSG_ERR, "Cannot open audio device.\n");
-		return false;
+	int driver = -1;
+	if (player->settings->audioPipe) {
+		// using audio pipe
+		struct stat st;
+		if (stat (player->settings->audioPipe, &st)) {
+			BarUiMsg (player->settings, MSG_ERR, "Cannot stat audio pipe file.\n");
+			return false;
+		}
+		if (!S_ISFIFO (st.st_mode)) {
+			BarUiMsg (player->settings, MSG_ERR, "File is not a pipe, error.\n");
+			return false;
+		}
+		driver = ao_driver_id ("raw");
+		if ((player->aoDev = ao_open_file(driver, player->settings->audioPipe, 1, &aoFmt, NULL)) == NULL) {
+			BarUiMsg (player->settings, MSG_ERR, "Cannot open audio pipe file.\n");
+			return false;
+		}
+	} else {
+		// use driver from libao configuration
+		driver = ao_default_driver_id ();
+		if ((player->aoDev = ao_open_live (driver, &aoFmt, NULL)) == NULL) {
+			BarUiMsg (player->settings, MSG_ERR, "Cannot open audio device.\n");
+			return false;
+		}
 	}
 
 	return true;
